@@ -23,6 +23,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #define alloca __builtin_alloca
 #elif defined(sparc)
 #include <alloca.h>
+extern "C" void *__builtin_alloca(...);
 #elif defined(_AIX)
 #pragma alloca
 #else
@@ -254,9 +255,18 @@ Key_List::read_keys (void)
       /* Hash table this number of times larger than keyword number. */
       int table_size = (list_len = total_keys) * TABLE_MULTIPLE;
 
+#ifdef __GNUC__
       /* By allocating the memory here we save on dynamic allocation overhead. 
          Table must be a power of 2 for the hash function scheme to work. */
       List_Node *table[POW (table_size)];
+#else
+      // Note: we don't use new, because that invokes a custom operator new.
+      int malloc_size = table_size * sizeof(List_Node*);
+      if (malloc_size == 0) malloc_size = 1;
+      List_Node **table = (List_Node**)malloc(malloc_size);
+      if (table == NULL)
+	abort ();
+#endif
 
       /* Make large hash table for efficiency. */
       Hash_Table found_link (table, table_size);
@@ -290,10 +300,16 @@ Key_List::read_keys (void)
             trail = temp;
             
           /* Update minimum and maximum keyword length, if needed. */
-          max_key_len >?= temp->length;
-          min_key_len <?= temp->length;
+	  if (max_key_len < temp->length)
+	    max_key_len = temp->length;
+	  if (min_key_len > temp->length)
+	    min_key_len = temp->length;
         }
 
+#ifndef __GNUC__
+      free (table);
+#endif
+
       /* Exit program if links exists and option[DUP] not set, since we can't continue */
       if (total_duplicates)
         report_error (option[DUP]
@@ -849,13 +865,25 @@ Key_List::output_lookup_array (void)
     {
       const int DEFAULT_VALUE = -1;
 
-      struct 
+      struct duplicate_entry
         {
           int hash_value;       /* Hash value for this particular duplicate set. */
           int index;            /* Index into the main keyword storage array. */
           int count;            /* Number of consecutive duplicates at this index. */
-        } duplicates[total_duplicates], *dup_ptr = duplicates;
-      int lookup_array[max_hash_value + 1], *lookup_ptr = lookup_array + max_hash_value + 1;
+        };
+#ifdef __GNUC__
+      duplicate_entry duplicates[total_duplicates];
+      int lookup_array[max_hash_value + 1];
+#else
+      // Note: we don't use new, because that invokes a custom operator new.
+      duplicate_entry *duplicates = (duplicate_entry*)
+	malloc (total_duplicates * sizeof(duplicate_entry));
+      int *lookup_array = (int*)malloc(sizeof(int) * (max_hash_value + 1));
+      if (duplicates == NULL || lookup_array == NULL)
+	abort();
+#endif
+      duplicate_entry *dup_ptr = duplicates;
+      int *lookup_ptr = lookup_array + max_hash_value + 1;
 
       while (lookup_ptr > lookup_array)
         *--lookup_ptr = DEFAULT_VALUE;
@@ -869,7 +897,15 @@ Key_List::output_lookup_array (void)
           if (!temp->link &&
               (!temp->next || hash_value != temp->next->hash_value))
             continue;
-          *dup_ptr = (typeof (*dup_ptr)) { hash_value, temp->index, 1 };
+#ifdef __GNUC__
+          *dup_ptr = (duplicate_entry) { hash_value, temp->index, 1 };
+#else
+	  duplicate_entry _dups;
+	  _dups.hash_value = hash_value;
+	  _dups.index = temp->index;
+	  _dups.count = 1;
+	  *dup_ptr = _dups;
+#endif
           
           for (List_Node *ptr = temp->link; ptr; ptr = ptr->link)
             {
@@ -930,9 +966,13 @@ Key_List::output_lookup_array (void)
         }
 
       int max = INT_MIN;
-
-      for (lookup_ptr = lookup_array + max_hash_value + 1; lookup_ptr > lookup_array; max >?= abs (*--lookup_ptr))
-        ;
+      lookup_ptr = lookup_array + max_hash_value + 1;
+      while (lookup_ptr > lookup_array)
+	{
+	  int val = abs (*--lookup_ptr);
+	  if (max < val)
+	    max = val;
+        }
 
       char *indent = option[GLOBAL] ? "" : "  ";
       printf ("%sstatic %s%s lookup[] =\n%s%s{\n      ", indent, option[CONST] ? "const " : "",
@@ -955,6 +995,10 @@ Key_List::output_lookup_array (void)
         printf ("%*d,%s", field_width, *lookup_ptr, ++column % (max_column - 1) ? "" : "\n      ");
 
       printf ("\n%s%s};\n\n", indent, indent);
+#ifndef __GNUC__
+      free (duplicates);
+      free (lookup_array);
+#endif
     }
 }
 /* Generates C code to perform the keyword lookup. */
